"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _default() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var pluginOptions = args[1];
  var options = Object.assign({
    extractedFile: './src/translations/aggregated.json',
    langFiles: [{
      path: './src/translations/ru.json',
      cleanUpNewMessages: false
    }, {
      path: './src/translations/en.json',
      cleanUpNewMessages: true
    }]
  }, pluginOptions);
  /**
   * @returns {Array}
   */

  function getTargetFileContent() {
    var doesFileExists = _fs.default.existsSync(options.extractedFile);

    if (!doesFileExists) {
      return [];
    }

    var result = JSON.parse(_fs.default.readFileSync(options.extractedFile));
    return Array.isArray(result) ? result : [];
  }
  /**
   * @returns {Object.<string, string>}
   */


  function getTargetLangFileContent(path) {
    var doesFileExists = _fs.default.existsSync(path);

    if (!doesFileExists) {
      return {};
    }

    return Object.assign({}, JSON.parse(_fs.default.readFileSync(path)));
  }
  /**
   * @param {{id: string}[]} existing
   * @param {{id: string}[]} descriptors
   *
   * @returns {{id: string}[]}
   */


  function mergeDescriptors(existing, descriptors) {
    return existing // Remove from existing all messages with id, that used in descriptors
    .filter(function (descriptor) {
      return descriptors.findIndex(function (_ref) {
        var id = _ref.id;
        return id === descriptor.id;
      }) === -1;
    }) // Merge with new descriptors array
    .concat(descriptors).sort(function (a, b) {
      return a.id < b.id ? -1 : a.id > b.id ? 1 : 0;
    });
  }

  function mergeMessages(cleanUpNewMessages, oldMessages, newMessages) {
    if (!cleanUpNewMessages) {
      return Object.assign(oldMessages, newMessages);
    }

    var result = Object.assign({}, oldMessages);
    Object.keys(newMessages).forEach(function (id) {
      if (!result[id]) {
        result[id] = '';
      }
    });
    return result;
  }

  return {
    post: function post(file) {
      var _file$metadata$react, _file$metadata$react$;

      if (!((_file$metadata$react = file.metadata['react-intl']) === null || _file$metadata$react === void 0 ? void 0 : (_file$metadata$react$ = _file$metadata$react.messages) === null || _file$metadata$react$ === void 0 ? void 0 : _file$metadata$react$.length)) {
        return;
      }

      var descriptors = file.metadata['react-intl'].messages;

      if (!Array.isArray(descriptors)) {
        return;
      }

      var existingDescriptors = getTargetFileContent();
      var resultDescriptors = mergeDescriptors(existingDescriptors, descriptors);

      if (!(resultDescriptors === null || resultDescriptors === void 0 ? void 0 : resultDescriptors.length)) {
        return;
      }

      _fs.default.writeFileSync(options.extractedFile, JSON.stringify(resultDescriptors, null, 2));

      var messages = {};
      resultDescriptors.forEach(function (_ref2) {
        var id = _ref2.id,
            defaultMessage = _ref2.defaultMessage;
        messages[id] = defaultMessage || '';
      });
      options.langFiles.forEach(function (_ref3) {
        var path = _ref3.path,
            cleanUpNewMessages = _ref3.cleanUpNewMessages;
        var resultMessages = mergeMessages(cleanUpNewMessages, getTargetLangFileContent(path), messages);

        _fs.default.writeFileSync(path, JSON.stringify(resultMessages, null, 2));
      });
    }
  };
}